@@ -89,6 +89,18 @@ namespace TRACT{
     m_x_s_init=0;
     m_y_s_init=0;
     m_z_s_init=0;
+
+    // create rotdir for cases where seeding from a freesurfer mesh
+    Matrix F(3,3),u(3,3),v(3,3);
+    DiagonalMatrix d(3);
+    F << -m_Seeds_to_DTI(1,1) << m_Seeds_to_DTI(1,3) << -m_Seeds_to_DTI(1,2)
+      << -m_Seeds_to_DTI(2,1) << m_Seeds_to_DTI(2,3) << -m_Seeds_to_DTI(2,2)
+      << -m_Seeds_to_DTI(3,1) << m_Seeds_to_DTI(3,3) << -m_Seeds_to_DTI(3,2);
+    
+    SVD(F*F.t(),d,u,v);
+    m_rotdir.ReSize(3,3);
+    m_rotdir = (u*sqrt(d)*v.t()).i()*F;
+
   }
   
   
@@ -105,15 +117,22 @@ namespace TRACT{
     ColumnVector xyz_dti;
     ColumnVector th_ph_f;
     float xst,yst,zst,x,y,z,tmp2;
+
+
     xyz_dti=vox_to_vox(xyz_seeds,dim_seeds,vols.dimensions(),m_Seeds_to_DTI);    
+
     xst=xyz_dti(1);yst=xyz_dti(2);zst=xyz_dti(3);
     m_path.clear();
     x=xst;y=yst;z=zst;
     m_part.change_xyz(x,y,z);
-    if(opts.meshfile.value()!="")
-      m_part.set_dir(dir(1),dir(2),dir(3));//Set the start dir so that we track inwards from cortex
-    
 
+    if(opts.meshfile.value()!=""){
+      // rotate dir using seeds_to_dti*mm_to_vox
+      ColumnVector rotdir(3);
+      rotdir = m_rotdir*dir;
+      m_part.set_dir(rotdir(1),rotdir(2),rotdir(3));//Set the start dir so that we track inwards from cortex
+    }
+    
     int partlength=0;
     bool rubbish_passed=false;
     bool stop_flag=false;
@@ -184,8 +203,8 @@ namespace TRACT{
 	  if(m_stop(x_s,y_s,z_s)!=0){
 	    stop_flag=true;
 	  }
-	  else
-	    if(stop_flag)break;
+	  //else
+	  if(stop_flag)break;
 	}	  
 	  
 	if(opts.skipmask.value() == ""){
@@ -280,6 +299,33 @@ namespace TRACT{
       m_seedcounts.push_back(tmpint);
       //m_particle_numbers.push_back(tmpvec);
     }
+
+    // where we save the seed counts in text files
+    if(opts.seedcountastext.value()){
+      
+      int numseeds=0;
+      if(opts.meshfile.value()==""){
+	for(int Wz=m_seeds.minz();Wz<=m_seeds.maxz();Wz++)
+	  for(int Wy=m_seeds.miny();Wy<=m_seeds.maxy();Wy++)
+	    for(int Wx=m_seeds.minx();Wx<=m_seeds.maxx();Wx++)
+	      if(m_seeds.value(Wx,Wy,Wz)!=0)
+		numseeds++;
+      }
+      else{
+	ifstream fs(opts.seedfile.value().c_str());
+	if(fs){
+	  char buffer[1024];
+	  fs.getline(buffer,1024);
+	  fs >>numseeds;
+	  cout<<numseeds<<endl;
+	}
+      }
+
+      m_SeedCountMat.ReSize(numseeds,m_targetmasknames.size());
+      m_SeedCountMat=0;
+      m_SeedRow=1;
+    }
+
   }
   
 
@@ -312,18 +358,30 @@ namespace TRACT{
   }
   
   void Counter::initialise_matrix2(){
+    
     m_Conrow2=0;
     read_volume(m_lrmask,opts.lrmask.value());
     m_beenhere2.reinitialize(m_lrmask.xsize(),m_lrmask.ysize(),m_lrmask.zsize());
     m_lrdim.ReSize(3);
     m_lrdim<<m_lrmask.xdim()<<m_lrmask.ydim()<<m_lrmask.zdim();
     int numseeds=0,numnz=0;
+    if(opts.meshfile.value()==""){
     for(int Wz=m_seeds.minz();Wz<=m_seeds.maxz();Wz++)
       for(int Wy=m_seeds.miny();Wy<=m_seeds.maxy();Wy++)
 	for(int Wx=m_seeds.minx();Wx<=m_seeds.maxx();Wx++)
 	  if(m_seeds.value(Wx,Wy,Wz)!=0)
 	    numseeds++;
-    
+    }
+    else{
+      ifstream fs(opts.seedfile.value().c_str());
+      if(fs){
+	char buffer[1024];
+	fs.getline(buffer,1024);
+	fs >>numseeds;
+	cout<<"numseeds="<<numseeds<<endl;
+      }
+      
+    }
     for(int Wz=m_lrmask.minz();Wz<=m_lrmask.maxz();Wz++)
       for(int Wy=m_lrmask.miny();Wy<=m_lrmask.maxy();Wy++)
 	for(int Wx=m_lrmask.minx();Wx<=m_lrmask.maxx();Wx++)
@@ -339,7 +397,10 @@ namespace TRACT{
       cerr<<" Although, it must stay in line with the seeds"<<endl;
       exit(-1);
     }
+
     m_ConMat2.reinitialize(numseeds,numnz,1);
+    OUT(m_ConMat2.xsize());
+
     m_CoordMat2.reinitialize(numseeds,3,1);
     m_CoordMat_tract2.reinitialize(numnz,3,1);
     
@@ -358,7 +419,7 @@ namespace TRACT{
 	    m_CoordMat_tract2(mytrow,2,0)=Wz;
 	    mytrow++;
 	  }
-      
+    
     int myrow=0;
     for(int Wz=m_seeds.minz();Wz<=m_seeds.maxz();Wz++)
       for(int Wy=m_seeds.miny();Wy<=m_seeds.maxy();Wy++)
@@ -395,6 +456,9 @@ namespace TRACT{
     if(opts.matrix2out.value()){
       next_matrix2_row();
     }
+    if(opts.seedcountastext.value()){
+      m_SeedRow++;
+    }
   }
   
     
@@ -468,6 +532,10 @@ namespace TRACT{
 	    m_seedcounts[m](xseedvox,yseedvox,zseedvox)=m_seedcounts[m](xseedvox,yseedvox,zseedvox)+1;
 	    m_targflags[m]=1;
 	    //m_particle_numbers[m].push_back(particle_number);
+
+	    if(opts.seedcountastext.value())
+	      m_SeedCountMat(m_SeedRow,m+1) += 1;
+	    
 	  }
 	}
       }
@@ -484,6 +552,10 @@ namespace TRACT{
 	    m_seedcounts[m](xseedvox,yseedvox,zseedvox)+=(int)d;
 	    m_targflags[m]=1;
 	    //m_particle_numbers[m].push_back(particle_number);
+	    
+	    if(opts.seedcountastext.value())
+	      m_SeedCountMat(m_SeedRow,m+1) += d;
+
 	  }
 	}
 	
@@ -522,7 +594,9 @@ namespace TRACT{
     if(!opts.pathdist.value())
       for(unsigned int i=0;i<path.size();i++){
 	ColumnVector xyz_seeds=path[i];
+	//do something here
 	ColumnVector xyz_lr=vox_to_vox(xyz_seeds,m_seedsdim,m_lrdim,m_I);
+	
 	int x_lr=int(round(float(xyz_lr(1)))),y_lr=int(round(float(xyz_lr(2)))),z_lr=int(round(float(xyz_lr(3))));
 	int Concol2=m_lookup2(x_lr,y_lr,z_lr,0);
 	if(Concol2!=0){
@@ -557,7 +631,9 @@ namespace TRACT{
     if(forwardflag){
       for(unsigned int i=0;i<m_path.size();i++){
 	ColumnVector xyz_seeds=m_path[i];
+
 	ColumnVector xyz_lr=vox_to_vox(xyz_seeds,m_seedsdim,m_lrdim,m_I);
+
 	int x_lr=int(round(float(xyz_lr(1)))),y_lr=int(round(float(xyz_lr(2)))),z_lr=int(round(float(xyz_lr(3))));
 	m_beenhere2(x_lr,y_lr,z_lr)=0;
       }
@@ -566,7 +642,9 @@ namespace TRACT{
       const vector<ColumnVector>& path=m_stline.get_path_ref();
       for(unsigned int i=0;i<path.size();i++){
 	ColumnVector xyz_seeds=path[i];
+
 	ColumnVector xyz_lr=vox_to_vox(xyz_seeds,m_seedsdim,m_lrdim,m_I);
+
 	int x_lr=int(round(float(xyz_lr(1)))),y_lr=int(round(float(xyz_lr(2)))),z_lr=int(round(float(xyz_lr(3))));
 	m_beenhere2(x_lr,y_lr,z_lr)=0;
       }  
@@ -641,6 +719,11 @@ namespace TRACT{
       
       save_volume(m_seedcounts[m],logger.appendDir("seeds_to_"+tmpname));
     }
+
+    if(opts.seedcountastext.value()){
+      write_ascii_matrix(m_SeedCountMat,logger.appendDir("matrix_seeds_to_all_targets"));
+    }
+
   }
     
   // the following is a helper function for save_matrix*
@@ -665,14 +748,56 @@ namespace TRACT{
   }
 
   void Counter::save_matrix2(){
-    save_volume(m_ConMat2,logger.appendDir("fdt_matrix2"));
-    applycoordchange(m_CoordMat2, m_seeds.niftivox2newimagevox_mat().i());
-    save_volume(m_CoordMat2,logger.appendDir("coords_for_fdt_matrix2"));
-    applycoordchange(m_CoordMat2, m_seeds.niftivox2newimagevox_mat());
-    applycoordchange(m_CoordMat_tract2, m_lrmask.niftivox2newimagevox_mat().i());
-    save_volume(m_CoordMat_tract2,logger.appendDir("tract_space_coords_for_fdt_matrix2"));
-    applycoordchange(m_CoordMat_tract2, m_lrmask.niftivox2newimagevox_mat());
-    save_volume4D(m_lookup2,logger.appendDir("lookup_tractspace_fdt_matrix2"));
+    if(!opts.splitmatrix2.value()){
+      save_volume(m_ConMat2,logger.appendDir("fdt_matrix2"));
+      applycoordchange(m_CoordMat2, m_seeds.niftivox2newimagevox_mat().i());
+      save_volume(m_CoordMat2,logger.appendDir("coords_for_fdt_matrix2"));
+      applycoordchange(m_CoordMat2, m_seeds.niftivox2newimagevox_mat());
+      applycoordchange(m_CoordMat_tract2, m_lrmask.niftivox2newimagevox_mat().i());
+      save_volume(m_CoordMat_tract2,logger.appendDir("tract_space_coords_for_fdt_matrix2"));
+      applycoordchange(m_CoordMat_tract2, m_lrmask.niftivox2newimagevox_mat());
+      save_volume4D(m_lookup2,logger.appendDir("lookup_tractspace_fdt_matrix2"));
+    }
+    else{
+      cout << "saving matrix2 into splitted files" << endl;
+
+      int nsplits = 10;
+      while( float(m_ConMat2.xsize()/nsplits) >= 32767 ){
+	  nsplits++;
+      }
+
+      int nrows = std::floor(float(m_ConMat2.xsize()/nsplits))+1;
+      volume<int> tmpmat;
+
+      applycoordchange(m_CoordMat2, m_seeds.niftivox2newimagevox_mat().i());
+
+      for(int i=1;i<=nsplits;i++){
+	int first_row = (i-1)*nrows+1;
+	int last_row  = i*nrows > m_ConMat2.xsize() ? m_ConMat2.xsize() : i*nrows;
+	if(first_row > m_ConMat2.xsize()) break;
+
+	// set limits
+	m_ConMat2.setROIlimits(first_row-1,m_ConMat2.miny(),m_ConMat2.minz(),last_row-1,m_ConMat2.maxy(),m_ConMat2.maxz());
+	m_ConMat2.activateROI();
+	tmpmat = m_ConMat2.ROI();
+	save_volume(tmpmat,logger.appendDir("fdt_matrix2_"+num2str(i)));
+
+	m_CoordMat2.setROIlimits(first_row-1,m_CoordMat2.miny(),m_CoordMat2.minz(),last_row-1,m_CoordMat2.maxy(),m_CoordMat2.maxz());
+	m_CoordMat2.activateROI();
+	tmpmat = m_CoordMat2.ROI();
+	save_volume(tmpmat,logger.appendDir("coords_for_fdt_matrix2_"+num2str(i)));
+
+
+      }
+
+      applycoordchange(m_CoordMat_tract2, m_lrmask.niftivox2newimagevox_mat());
+      save_volume4D(m_lookup2,logger.appendDir("lookup_tractspace_fdt_matrix2"));
+
+      applycoordchange(m_CoordMat2, m_seeds.niftivox2newimagevox_mat());
+      applycoordchange(m_CoordMat_tract2, m_lrmask.niftivox2newimagevox_mat().i());
+      save_volume(m_CoordMat_tract2,logger.appendDir("tract_space_coords_for_fdt_matrix2"));
+
+    }
   }
   
   int Seedmanager::run(const float& x,const float& y,const float& z,bool onewayonly, int fibst){
@@ -709,23 +834,26 @@ namespace TRACT{
       
       m_stline.reset();
       bool forwardflag=false,backwardflag=false;
+      bool counted=false;
       if(!onewayonly){
 	if(m_stline.streamline(x,y,z,m_seeddims,fibst,dir)){ //returns whether to count the streamline or not
 	  forwardflag=true;
 	  m_counter.store_path();
 	  m_counter.count_streamline();
-	  nlines++;
+	  nlines++;counted=true;
 	}
 	m_stline.reverse();
       }
       if(m_stline.streamline(x,y,z,m_seeddims,fibst,dir)){
 	backwardflag=true;
 	m_counter.count_streamline();
-	//nlines++; //count twice ?
+	if(!counted)nlines++; // the other half has is counted here
+
       }
      
       m_counter.clear_streamline(forwardflag,backwardflag); 
     }
+
     m_counter.count_seed();
     
     return nlines;
