@@ -5,7 +5,7 @@
 
     Christian F. Beckmann, FMRIB Image Analysis Group
     
-    Copyright (C) 1999-2007 University of Oxford */
+    Copyright (C) 1999-2008 University of Oxford */
 
 /*  Part of FSL - FMRIB's Software Library
     http://www.fmrib.ox.ac.uk/fsl
@@ -528,7 +528,7 @@ namespace Melodic{
       }
     }
 
-    dlm += Identity(nummix)*dlm.Maximum();
+    dlm += IdentityMatrix(nummix)*dlm.Maximum();
 
     int i,j;
     float val;
